use palette::FromColor;
use std::collections::HashMap;
use std::str::FromStr;

pub use setting::SwatchSchemeSetting;
use swatch::Swatch;
use wasm_bindgen::prelude::*;

use crate::{errors, parse_to_oklch};

use super::SchemeExport;

mod setting;
mod swatch;

#[derive(Debug, Clone)]
pub struct SwatchScheme {
    light: HashMap<String, Swatch>,
    dark: HashMap<String, Swatch>,
}

#[derive(Debug, Clone)]
#[wasm_bindgen(getter_with_clone)]
pub struct SwatchEntry {
    pub name: String,
    pub color: String,
}

impl SwatchScheme {
    pub fn new(
        colors: Vec<SwatchEntry>,
        setting: SwatchSchemeSetting,
    ) -> Result<Self, errors::ColorError> {
        let mut light = HashMap::new();
        let mut dark = HashMap::new();

        for entry in colors {
            let color = parse_to_oklch!(&entry.color);
            let darken_color = color * setting.dark_convert;
            light.insert(entry.name.clone(), Swatch::new(&color, &setting));
            dark.insert(entry.name, Swatch::new(&darken_color, &setting));
        }

        Ok(Self { light, dark })
    }

    pub fn swatches(&self) -> HashMap<String, HashMap<String, Vec<String>>> {
        let mut light_swatches = HashMap::new();
        let mut dark_swatches = HashMap::new();

        for (name, swatch) in &self.light {
            light_swatches.insert(name.clone(), swatch.swtch_hex());
        }
        for (name, swatch) in &self.dark {
            dark_swatches.insert(name.clone(), swatch.swtch_hex());
        }

        HashMap::from([
            ("light".to_string(), light_swatches),
            ("dark".to_string(), dark_swatches),
        ])
    }
}

impl SchemeExport for SwatchScheme {
    fn output_css_variables(&self) -> String {
        let mut variables = Vec::new();

        for (name, swatch) in &self.light {
            variables.extend(swatch.to_css_variables("light", name));
        }
        for (name, swatch) in &self.dark {
            variables.extend(swatch.to_css_variables("dark", name));
        }

        variables.join("\n")
    }

    fn output_scss_variables(&self) -> String {
        let mut variables = Vec::new();

        for (name, swatch) in &self.light {
            variables.extend(swatch.to_scss_variables("light", name));
        }
        for (name, swatch) in &self.dark {
            variables.extend(swatch.to_scss_variables("dark", name));
        }

        variables.join("\n")
    }

    fn output_javascript_object(&self) -> String {
        let mut object = Vec::new();
        object.push("{".to_string());

        object.push("  light: {".to_string());
        for (name, swatch) in &self.light {
            object.extend(
                swatch
                    .to_javascript_fields("light", name)
                    .iter()
                    .map(|s| format!("    {}", s)),
            );
        }
        object.push("  },".to_string());

        object.push("  dark: {".to_string());
        for (name, swatch) in &self.dark {
            object.extend(
                swatch
                    .to_javascript_fields("dark", name)
                    .iter()
                    .map(|s| format!("    {}", s)),
            );
        }
        object.push("  },".to_string());

        object.push("}".to_string());

        object.join("\n")
    }
}
